@@ -33,17 +33,21 @@ import (
 
 type CoreGrubImage struct {
 	BaseImage
+
+	legacyGrub bool
 }
 
-func NewCoreGrubImage(location string, size int64, hw HardwareDescription, oem OemDescription) *CoreGrubImage {
+func NewCoreGrubImage(location string, size int64, rootSize int, hw HardwareDescription, oem OemDescription, updateGrub bool) *CoreGrubImage {
 	return &CoreGrubImage{
-		BaseImage{
+		BaseImage: BaseImage{
 			location:  location,
 			size:      size,
+			rootSize:  rootSize,
 			hardware:  hw,
 			oem:       oem,
 			partCount: 5,
 		},
+		legacyGrub: updateGrub,
 	}
 }
 
@@ -71,8 +75,8 @@ func (img *CoreGrubImage) Partition() error {
 
 	parted.addPart(grubLabel, "", fsNone, 4)
 	parted.addPart(bootLabel, bootDir, fsFat32, 64)
-	parted.addPart(systemALabel, systemADir, fsExt4, 1024)
-	parted.addPart(systemBLabel, systemBDir, fsExt4, 1024)
+	parted.addPart(systemALabel, systemADir, fsExt4, img.rootSize)
+	parted.addPart(systemBLabel, systemBDir, fsExt4, img.rootSize)
 	parted.addPart(writableLabel, writableDir, fsExt4, -1)
 
 	parted.setBoot(2)
@@ -83,7 +87,19 @@ func (img *CoreGrubImage) Partition() error {
 	return parted.create(img.location)
 }
 
-func (img *CoreGrubImage) SetupBoot(oemRootPath string) error {
+func (img *CoreGrubImage) SetupBoot() error {
+	if !img.legacyGrub {
+		// destinations
+		bootPath := filepath.Join(img.baseMount, string(bootDir), "EFI", "ubuntu", "grub")
+		if err := img.GenericBootSetup(bootPath); err != nil {
+			return err
+		}
+	}
+
+	return img.setupGrub()
+}
+
+func (img *CoreGrubImage) setupGrub() error {
 	for _, dev := range []string{"dev", "proc", "sys"} {
 		src := filepath.Join("/", dev)
 		dst := filepath.Join(img.System(), dev)
@@ -135,7 +151,6 @@ func (img *CoreGrubImage) SetupBoot(oemRootPath string) error {
 	}
 
 	bootGrubDir := filepath.Join(img.System(), "boot", "grub")
-
 	if err := bindMount(efiGrubDir, bootGrubDir); err != nil {
 		return err
 	}
@@ -178,6 +193,16 @@ func (img *CoreGrubImage) SetupBoot(oemRootPath string) error {
 		return err
 	}
 
+	if img.legacyGrub {
+		if err := img.updateGrub(); err != nil {
+			return err
+		}
+	}
+
+	return nil
+}
+
+func (img *CoreGrubImage) updateGrub() error {
 	// ensure we run not into recordfail issue
 	grubDir := filepath.Join(img.System(), "etc", "default", "grub.d")
 	if err := os.MkdirAll(grubDir, 0755); err != nil {
@@ -202,10 +227,6 @@ func (img *CoreGrubImage) SetupBoot(oemRootPath string) error {
 	return nil
 }
 
-func (img *CoreGrubImage) FlashExtra(oemRootPath, devicePart string) error {
-	return nil
-}
-
 func bindMount(src, dst string) error {
 	if out, err := exec.Command("mount", "--bind", src, dst).CombinedOutput(); err != nil {
 		return fmt.Errorf("issues while bind mounting: %s", out)
